# Generated by Django 2.2.15 on 2020-09-09 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0004_adding_learner_data_to_canvas'),
    ]

    operations = [
        migrations.AlterField(
            model_name='canvaslearnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.CharField(help_text='Represents the canvas representation of a timestamp: yyyy-mm-dd, which is always 10 characters.', max_length=10),
        ),
        migrations.AlterField(
            model_name='canvaslearnerdatatransmissionaudit',
            name='course_completed',
            field=models.BooleanField(default=False, help_text="The learner's course completion status transmitted to Canvas."),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Content metadata exporter for Degreed.
"""

from logging import getLogger

from enterprise.utils import get_closest_course_run, get_course_run_duration_info, parse_datetime_handle_invalid
from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter
from integrated_channels.utils import generate_formatted_log, get_image_url, strip_html_tags

LOGGER = getLogger(__name__)


class Degreed2ContentMetadataExporter(ContentMetadataExporter):
    """
    Degreed2 implementation of ContentMetadataExporter.
    """

    CHUNK_PAGE_LENGTH = 1000
    SHORT_STRING_LIMIT = 255
    LONG_STRING_LIMIT = 2000

    DATA_TRANSFORM_MAPPING = {
        'title': 'title',
        'summary': 'description',
        'image-url': 'image',
        'url': 'enrollment_url',
        'language': 'content_language',
        'external-id': 'key',
        'duration': 'duration',
        'duration-type': 'duration_type',
    }

    def transform_duration_type(self, content_metadata_item):  # pylint: disable=unused-argument
        return 'Days'

    def transform_duration(self, content_metadata_item):
        """
        Returns: duration in days
        """
        if content_metadata_item.get('content_type') == 'courserun':
            start = content_metadata_item.get('start')
            end = content_metadata_item.get('end')
        elif content_metadata_item.get('content_type') == 'course':
            course_runs = content_metadata_item.get('course_runs')
            if course_runs:
                course_run = get_closest_course_run(course_runs)
                if course_run:
                    start = course_run.get('start')
                    end = course_run.get('end')
                else:
                    LOGGER.error(
                        generate_formatted_log(
                            self.enterprise_configuration.channel_code(),
                            self.enterprise_configuration.enterprise_customer.uuid,
                            None,
                            None,
                            'Cannot find a courserun, so duration being returned 0. '
                            f'Course item was {content_metadata_item} '
                        )
                    )
                    return 0
            else:
                LOGGER.error(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        'Cannot find even a single courserun, so duration being returned 0. '
                        f'Course item was {content_metadata_item} '
                    )
                )
                return 0
        else:
            return 0

        start_date = parse_datetime_handle_invalid(start)
        end_date = parse_datetime_handle_invalid(end)
        if not start_date or not end_date:
            LOGGER.error(
                generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    self.enterprise_configuration.enterprise_customer.uuid,
                    None,
                    None,
                    'Failed to find valid start/end dates, so duration is going to be set to 0. '
                    f'Course item was {content_metadata_item} '
                    f'Parsed Start was: {start_date}, End was: {end_date}'
                )
            )
            return 0
        return (end_date - start_date).days

    def transform_description(self, content_metadata_item):
        """
        Return the transformed version of the course description.

        We choose one value out of the course's full description, short description, and title
        depending on availability and length limits.
        """
        course_runs = content_metadata_item.get('course_runs')
        duration_info = get_course_run_duration_info(
            get_closest_course_run(course_runs)
        ) if course_runs else ''
        full_description = content_metadata_item.get('full_description') or ''
        if full_description and 0 < len(full_description + duration_info) <= self.LONG_STRING_LIMIT:
            description = full_description
        else:
            description = content_metadata_item.get('short_description') or content_metadata_item.get('title') or ''
        if description:
            description = "{duration_info}{description}".format(duration_info=duration_info, description=description)
        return strip_html_tags(description)

    def transform_courserun_content_language(self, content_metadata_item):
        """
        Return the ISO 639-1 language code that Degreed expects for course runs.

        Example:
            en-us -> en
            None -> en
        """
        code = content_metadata_item.get('content_language') or ''
        return code.split('-')[0] or 'en'

    def transform_content_language(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Return the ISO 639-1 language code that Degreed expects.

        Example:
            en-us -> en
            None -> en
        """
        # TODO: This needs to be implemented once we have richer data from the discovery service
        return 'en'

    def transform_image(self, content_metadata_item):
        """
        Return the image URI of the content item.
        """
        return get_image_url(content_metadata_item)

    def transform_program_key(self, content_metadata_item):
        """
        Return the identifier of the program content item.
        """
        return content_metadata_item['uuid']



"""
Django management command for creating EnterpriseCourseEnrollment records.
"""

import logging

from django.core.management import BaseCommand
from django.db import connection

from enterprise.models import (
    EnterpriseCourseEnrollment,
    EnterpriseCustomer,
    EnterpriseCustomerUser,
    EnterpriseEnrollmentSource,
)

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Creates EnterpriseCourseEnrollment records (if they do not already exist) for CourseEnrollment records
    that are associated with an enterprise user and a course run that exists in the enterprise's catalog.
    """
    help = 'Create EnterpriseCourseEnrollment records for CourseEnrollment records associated with an enterprise.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-e',
            '--enterprise_customer_uuid',
            action='store',
            dest='enterprise_customer_uuid',
            default=None,
            help='Run this command for only the given EnterpriseCustomer UUID.'
        )

    def handle(self, *args, **options):
        LOGGER.info("Command has started...")
        enterprise_customer_uuid_filter = options.get('enterprise_customer_uuid')
        records_created = 0
        records_failed = 0

        missing_enrollment_data = self._fetch_course_enrollment_data(
            enterprise_customer_uuid_filter
        )
        LOGGER.info('System has %s missing enrollments', len(missing_enrollment_data))
        for item in missing_enrollment_data:
            course_exist_in_catalog = False
            user_id = item['user_id']
            course_run_id = item['course_run_id']
            enterprise_customer_uuid = item['enterprise_customer_uuid']

            LOGGER.info(
                'Trying to create the enrollment for user [%s] in course [%s] for enterprise customer [%s]',
                user_id,
                course_run_id,
                enterprise_customer_uuid
            )

            enterprise_customer = EnterpriseCustomer.objects.get(uuid=enterprise_customer_uuid)

            try:
                LOGGER.info(
                    'Checking whether course [%s] exists in enterprise customer [%s] - [%s] catalog',
                    course_run_id,
                    enterprise_customer_uuid,
                    enterprise_customer.name
                )
                course_exist_in_catalog = enterprise_customer.catalog_contains_course(course_run_id)
            except Exception as exc:    # pylint: disable=broad-except
                records_failed += 1
                LOGGER.warning(
                    'Course [%s] does not exist in EnterpriseCustomer [%s] due to this exception: [%s]',
                    course_run_id,
                    enterprise_customer.uuid,
                    str(exc)
                )

            if course_exist_in_catalog:
                enterprise_customer_user = EnterpriseCustomerUser.objects.filter(
                    enterprise_customer=enterprise_customer_uuid,
                    user_id=user_id
                )

                # This is an extra check for preventing the exception.
                # We already have implemented the solution for this (soft deletion).
                if enterprise_customer_user.exists():
                    enterprise_customer_user = enterprise_customer_user.first()
                    __, created = EnterpriseCourseEnrollment.objects.get_or_create(
                        enterprise_customer_user=enterprise_customer_user,
                        course_id=course_run_id,
                        defaults={
                            'source': EnterpriseEnrollmentSource.get_source(
                                EnterpriseEnrollmentSource.MANAGEMENT_COMMAND
                            )
                        }
                    )
                    if created:
                        # if we have enrolled the user in a course then we should
                        # active this record and inactive all the other records.
                        enterprise_customer_user.active = True
                        enterprise_customer_user.save()
                        EnterpriseCustomerUser.inactivate_other_customers(user_id, enterprise_customer)

                        records_created += 1
                        LOGGER.info(
                            'EnterpriseCourseEnrollment created: EnterpriseCustomer [%s] - User [%s] - CourseRun [%s]',
                            enterprise_customer_uuid,
                            user_id,
                            course_run_id
                        )
                    else:
                        LOGGER.warning(
                            'EnterpriseCourseEnrollment exists: EnterpriseCustomer [%s] - User [%s] - CourseRun [%s]',
                            enterprise_customer_uuid,
                            user_id,
                            course_run_id
                        )
                else:
                    LOGGER.info(
                        'User [%s] is not linked with EnterpriseCustomer - [%s]',
                        user_id,
                        enterprise_customer.uuid,
                    )

        LOGGER.info('Created %s missing EnterpriseCourseEnrollments.', records_created)
        LOGGER.info('Exception raised for %s records.', records_failed)

    def _fetch_course_enrollment_data(self, enterprise_customer_uuid):
        """
        Return enterprise customer UUID/user_id/course_run_id triples which represent CourseEnrollment records
        which do not have a matching EnterpriseCourseEnrollment record.

        The query used below looks for CourseEnrollment records that are associated with enterprise
        learners where the enrollment data is after the creation of the link between the learner
        and the enterprise. It also excludes learners with edx.org email addresses in order to
        filter out test users.
        """
        LOGGER.info("Trying to fetch the data from Database with enterprise customer [%s]", enterprise_customer_uuid)
        query = '''
            SELECT
                au.id as user_id,
                ecu.enterprise_customer_id as enterprise_customer_uuid,
                sce.course_id as course_run_id
            FROM student_courseenrollment sce
            JOIN auth_user au
                ON au.id = sce.user_id
            JOIN enterprise_enterprisecustomeruser ecu
                ON ecu.user_id = au.id
            LEFT JOIN enterprise_enterprisecourseenrollment ece
                ON ece.enterprise_customer_user_id = ecu.id
                AND ece.course_id = sce.course_id
            WHERE
                ecu.linked = true
                AND ece.id IS NULL
                AND ecu.created <= sce.created
                AND au.email NOT LIKE '%@edx.org'
                {enterprise_customer_filter}
            ORDER BY sce.created;
        '''

        with connection.cursor() as cursor:
            if enterprise_customer_uuid:
                cursor.execute(
                    query.format(enterprise_customer_filter='AND ecu.enterprise_customer_id = %s'),
                    [enterprise_customer_uuid]
                )
            else:
                cursor.execute(
                    query.format(enterprise_customer_filter='')
                )

            return self._dictfetchall(cursor)

    def _dictfetchall(self, cursor):
        """ Return all rows from a cursor as a dict. """
        columns = [col[0] for col in cursor.description]
        return [
            dict(zip(columns, row))
            for row in cursor.fetchall()
        ]



"""
URLs for enterprise.
"""

from django.conf import settings
from django.conf.urls import include
from django.urls import re_path

from enterprise.constants import COURSE_KEY_URL_PATTERN
from enterprise.heartbeat.views import heartbeat
from enterprise.views import (
    EnterpriseLoginView,
    EnterpriseProxyLoginView,
    EnterpriseSelectionView,
    GrantDataSharingPermissions,
    RouterView,
)

ENTERPRISE_ROUTER = RouterView.as_view()

urlpatterns = [
    re_path(r'^enterprise/grant_data_sharing_permissions', GrantDataSharingPermissions.as_view(),
            name='grant_data_sharing_permissions'
            ),
    re_path(r'^enterprise/select/active', EnterpriseSelectionView.as_view(),
            name='enterprise_select_active'
            ),
    re_path(r'^enterprise/login', EnterpriseLoginView.as_view(),
            name='enterprise_slug_login'
            ),
    re_path(r'^enterprise/proxy-login', EnterpriseProxyLoginView.as_view(),
            name='enterprise_proxy_login'
            ),
    re_path(
        r'^enterprise/handle_consent_enrollment/(?P<enterprise_uuid>[^/]+)/course/{}/$'.format(
            settings.COURSE_ID_PATTERN
        ),
        ENTERPRISE_ROUTER,
        name='enterprise_handle_consent_enrollment'
    ),
    re_path(
        r'^enterprise/(?P<enterprise_uuid>[^/]+)/course/{}/enroll/$'.format(COURSE_KEY_URL_PATTERN),
        ENTERPRISE_ROUTER,
        name='enterprise_course_enrollment_page'
    ),
    re_path(
        r'^enterprise/(?P<enterprise_uuid>[^/]+)/course/{}/enroll/$'.format(settings.COURSE_ID_PATTERN),
        ENTERPRISE_ROUTER,
        name='enterprise_course_run_enrollment_page'
    ),
    re_path(r'^enterprise/(?P<enterprise_uuid>[^/]+)/program/(?P<program_uuid>[^/]+)/enroll/$', ENTERPRISE_ROUTER,
            name='enterprise_program_enrollment_page'
            ),
    re_path(r'^enterprise/api/', include('enterprise.api.urls'),
            name='enterprise_api'
            ),
    re_path(r'^enterprise/heartbeat/', heartbeat,
            name='enterprise_heartbeat',
            ),
]

# Because ROOT_URLCONF points here, we are including the urls from the other apps here for now.
urlpatterns += [
    re_path('', include('consent.urls'),
            name='consent'
            ),
    re_path(r'^cornerstone/', include('integrated_channels.cornerstone.urls'),
            name='cornerstone'
            ),
    re_path(r'^canvas/', include('integrated_channels.canvas.urls'),
            name='canvas',
            ),
    re_path(r'^blackboard/', include('integrated_channels.blackboard.urls'),
            name='blackboard',
            ),
    re_path(r'^enterprise_learner_portal/', include('enterprise_learner_portal.urls'),
            name='enterprise_learner_portal_api'
            ),
    re_path(r'^integrated_channels/api/', include('integrated_channels.api.urls')
            ),
]



# Generated by Django 3.2.10 on 2021-12-21 15:35

from django.db import migrations, models
import uuid


def create_uuid(apps, schema_editor):
    CanvasEnterpriseCustomerConfiguration = apps.get_model('canvas', 'CanvasEnterpriseCustomerConfiguration')
    for config in CanvasEnterpriseCustomerConfiguration.objects.all():
        config.uuid = uuid.uuid4()
        config.save()


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0012_alter_canvasenterprisecustomerconfiguration_enterprise_customer'),
    ]

    operations = [
        migrations.AddField(
            model_name='canvasenterprisecustomerconfiguration',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, help_text='A UUID for use in public-facing urls such as oauth state variables.', null=True),
        ),
        migrations.AddField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='uuid',
            field=models.UUIDField(db_index=True, default=uuid.uuid4, editable=False, help_text='A UUID for use in public-facing urls such as oauth state variables.'),
        ),
        migrations.RunPython(create_uuid, reverse_code=migrations.RunPython.noop),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, help_text='A UUID for use in public-facing urls such as oauth state variables.', unique=True),
        ),
    ]



"""
Management command for reverting revoked enrollment
related objects to a particular time.
"""


import logging
from datetime import datetime

from dateutil.tz import tzutc

from django.contrib import auth
from django.core.management.base import BaseCommand

from enterprise.models import EnterpriseCourseEnrollment, EnterpriseCustomer

log = logging.getLogger(__name__)
User = auth.get_user_model()


class Command(BaseCommand):
    """
    Management command for reverting revoked enrollment related objects to a particular time.

    Example usage:
        $ ./manage.py revert_enrollment_objects --year 2021 --month 11 --day 17 --enterprise-customer-name test-co
    """
    help = 'Reverts revoked enrollment related objects to a particular time.'

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            '--year',
            action='store',
            dest='year',
            default=9999,
            help='Year of date',
            type=int,
        )
        parser.add_argument(
            '--month',
            action='store',
            dest='month',
            default=1,
            help='Month of date',
            type=int,
        )
        parser.add_argument(
            '--day',
            action='store',
            dest='day',
            default=0,
            help='Date of date',
            type=int,
        )
        parser.add_argument(
            '--hour',
            action='store',
            dest='hour',
            default=0,
            help='Hour of date',
            type=int,
        )
        parser.add_argument(
            '--minute',
            action='store',
            dest='minute',
            default=0,
            help='Minute of date',
            type=int,
        )
        parser.add_argument(
            '--second',
            action='store',
            dest='second',
            default=0,
            help='Second of date',
            type=int,
        )

        parser.add_argument(
            '--enterprise-customer-name',
            action='store',
            dest='enterprise_customer_name',
            help='Enterprise customer name',
            type=int,
        )

    def revert_enrollment_objects(self, options):
        """
        Revert all EnterpriseCourseEnrollment, LicensedEnterpriseCourseEnrollment, and "student" CourseEnrollment
        objects to the date provided, using the history table IF is_revoked = True on LicensedEnterpriseCourseEnrollment
        """
        # e.g. datetime(2021, 11, 18, 0, 0, tzinfo=tzutc())
        time_to_revert_to = datetime(
            options['year'],
            options['month'],
            options['day'],
            options['hour'],
            options['minute'],
            options['second'],
            tzinfo=tzutc()
        )

        ec = EnterpriseCustomer.objects.get(name=options['enterprise_customer_name'])
        ecus = ec.enterprise_customer_users.all()

        for ecu in ecus:
            eces = EnterpriseCourseEnrollment.objects.filter(
                enterprise_customer_user=ecu,
                licensed_with__is_revoked=True,
                licensed_with__modified__gte=time_to_revert_to,
            )

            for enrollment in eces:
                student_course_enrollment = enrollment.course_enrollment
                student_course_enrollment.history.as_of(time_to_revert_to).save()

                licensed_enrollment = enrollment.licensed_with
                licensed_enrollment.history.as_of(time_to_revert_to).save()

                enrollment.history.as_of(time_to_revert_to).save()

    def handle(self, *args, **options):
        """
        Entry point for management command execution.
        """
        log.info('Begin reverting enrollment objects back!')

        self.revert_enrollment_objects(options)

        log.info('Sucessfully reverted enrollment objects back!')



# Generated by Django 2.2.24 on 2021-09-30 21:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0009_auto_20210923_1727'),
    ]

    operations = [
        migrations.CreateModel(
            name='CornerstoneCourseKey',
            fields=[
                ('internal_course_id', models.CharField(help_text='This is the edX course key that is used as a unique identifier.', max_length=255, primary_key=True, serialize=False)),
                ('external_course_id', models.CharField(help_text='This is the course key that is being sent to our partners.', max_length=255)),
            ],
        ),
    ]



"""
Database models for Enterprise Integrated Channel Canvas.
"""

import json
import uuid
from logging import getLogger

from simple_history.models import HistoricalRecords
from six.moves.urllib.parse import urljoin

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from integrated_channels.canvas.exporters.content_metadata import CanvasContentMetadataExporter
from integrated_channels.canvas.exporters.learner_data import CanvasLearnerExporter
from integrated_channels.canvas.transmitters.content_metadata import CanvasContentMetadataTransmitter
from integrated_channels.canvas.transmitters.learner_data import CanvasLearnerTransmitter
from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration
from integrated_channels.utils import is_valid_url

LOGGER = getLogger(__name__)
LMS_OAUTH_REDIRECT_URL = urljoin(settings.LMS_ROOT_URL, '/canvas/oauth-complete')


class CanvasEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
    """
    The Enterprise-specific configuration we need for integrating with Canvas.

    Based on: https://canvas.instructure.com/doc/api/file.oauth.html#oauth2-flow-3

    .. no_pii:
    """

    client_id = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client ID",
        help_text=_(
            "The API Client ID provided to edX by the enterprise customer to be used to make API "
            "calls to Canvas on behalf of the customer."
        )
    )

    client_secret = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client Secret",
        help_text=_(
            "The API Client Secret provided to edX by the enterprise customer to be used to make "
            " API calls to Canvas on behalf of the customer."
        )
    )

    canvas_account_id = models.BigIntegerField(
        null=True,
        blank=True,
        verbose_name="Canvas Account Number",
        help_text=_("Account number to use during api calls. Called account_id in canvas. "
                    " Required to create courses etc.")
    )

    canvas_base_url = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Canvas Base URL",
        help_text=_("The base URL used for API requests to Canvas, i.e. https://instructure.com.")
    )

    refresh_token = models.CharField(
        max_length=255,
        blank=True,
        verbose_name="Oauth2 Refresh Token",
        help_text=_("The refresh token provided by Canvas along with the access token request, used to "
                    "re-request the access tokens over multiple client sessions.")
    )

    # overriding base model field, to use chunk size 1 default
    transmission_chunk_size = models.IntegerField(
        default=1,
        help_text=_("The maximum number of data items to transmit to the integrated channel "
                    "with each request.")
    )

    uuid = models.UUIDField(
        unique=True,
        default=uuid.uuid4,
        editable=False,
        help_text=(
            "A UUID for use in public-facing urls such as oauth state variables."
        )
    )

    history = HistoricalRecords()

    class Meta:
        app_label = 'canvas'

    @property
    def oauth_authorization_url(self):
        """
        Returns: the oauth authorization url when the canvas_base_url and client_id are available.

        Args:
            obj: The instance of CanvasEnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        if self.canvas_base_url and self.client_id:
            return (f'{self.canvas_base_url}/login/oauth2/auth'
                    f'?redirect_uri={LMS_OAUTH_REDIRECT_URL}&'
                    f'response_type=code&'
                    f'client_id={self.client_id}&state={self.uuid}')
        else:
            return None

    @property
    def is_valid(self):
        """
        Returns whether or not the configuration is valid and ready to be activated

        Args:
            obj: The instance of CanvasEnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        missing_items = {'missing': []}
        incorrect_items = {'incorrect': []}
        if not self.client_id:
            missing_items.get('missing').append('client_id')
        if not self.client_secret:
            missing_items.get('missing').append('client_secret')
        if not self.canvas_base_url:
            missing_items.get('missing').append('canvas_base_url')
        if not self.canvas_account_id:
            missing_items.get('missing').append('canvas_account_id')
        if not self.refresh_token:
            missing_items.get('missing').append('refresh_token')
        if not is_valid_url(self.canvas_base_url):
            incorrect_items.get('incorrect').append('canvas_base_url')
        if len(self.display_name) > 30:
            incorrect_items.get('incorrect').append('display_name')
        return missing_items, incorrect_items

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<CanvasEnterpriseCustomerConfiguration for Enterprise {enterprise_name}>".format(
            enterprise_name=self.enterprise_customer.name
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @staticmethod
    def channel_code():
        """
        Returns an capitalized identifier for this channel class, unique among subclasses.
        """
        return 'CANVAS'

    def get_learner_data_transmitter(self):
        return CanvasLearnerTransmitter(self)

    def get_learner_data_exporter(self, user):
        return CanvasLearnerExporter(user, self)

    def get_content_metadata_exporter(self, user):
        return CanvasContentMetadataExporter(user, self)

    def get_content_metadata_transmitter(self):
        return CanvasContentMetadataTransmitter(self)


class CanvasLearnerAssessmentDataTransmissionAudit(models.Model):
    """
    The payload correlated to a courses subsection learner data we send to canvas at a given point in time for an
    enterprise course enrollment.

    .. no_pii:
    """
    canvas_user_email = models.CharField(
        max_length=255,
        blank=False,
        null=False
    )

    enterprise_course_enrollment_id = models.PositiveIntegerField(
        blank=False,
        null=False,
        db_index=True
    )

    course_id = models.CharField(
        max_length=255,
        blank=False,
        null=False,
        help_text=_("The course run's key which is used to uniquely identify the course for Canvas.")
    )

    subsection_id = models.CharField(
        max_length=255,
        blank=False,
        null=False,
        db_index=True,
        help_text=_("The course's subsections's key.")
    )

    grade_point_score = models.FloatField(
        blank=False,
        null=False,
        help_text=_("The amount of points that the learner scored on the subsection.")
    )

    grade_points_possible = models.FloatField(
        blank=False,
        null=False,
        help_text=_("The total amount of points that the learner could score on the subsection.")
    )

    # Request-related information.
    grade = models.FloatField(
        blank=False,
        null=False,
        help_text=_("The grade an enterprise learner received on the reported subsection.")
    )
    subsection_name = models.CharField(
        max_length=255,
        blank=False,
        help_text=_("The name given to the subsection being reported. Used for displaying on external LMS'.")
    )
    status = models.CharField(max_length=100, blank=False, null=False)
    error_message = models.TextField(blank=True)
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = 'canvas'

    def __str__(self):
        return (
            '<CanvasLearnerAssessmentDataTransmissionAudit {transmission_id} for enterprise enrollment {enrollment}, '
            'email {canvas_user_email}, and course {course_id}>'.format(
                transmission_id=self.id,
                enrollment=self.enterprise_course_enrollment_id,
                canvas_user_email=self.canvas_user_email,
                course_id=self.course_id
            )
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    def serialize(self, *args, **kwargs):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.

        # TODO: When we refactor to use a serialization flow consistent with how course metadata
        # is serialized, remove the serialization here and make the learner data exporter handle the work.
        """
        return json.dumps(self._payload_data(), sort_keys=True)

    def _payload_data(self):
        """
        Convert the audit record's fields into Canvas key/value pairs.
        """
        return dict(
            userID=self.canvas_user_email,
            courseID=self.course_id,
            grade=self.grade,
            subsectionID=self.subsection_id,
            points_possible=self.grade_points_possible,
            points_earned=self.grade_point_score,
            subsection_name=self.subsection_name,
        )


class CanvasLearnerDataTransmissionAudit(models.Model):
    """
    The payload we send to canvas at a given point in time for an enterprise course enrollment.

    .. no_pii:
    """
    canvas_user_email = models.CharField(
        max_length=255,
        blank=False,
        null=False
    )

    enterprise_course_enrollment_id = models.PositiveIntegerField(
        blank=False,
        null=False,
        db_index=True
    )

    course_id = models.CharField(
        max_length=255,
        blank=False,
        null=False,
        help_text=_("The course run's key which is used to uniquely identify the course for Canvas.")
    )

    course_completed = models.BooleanField(
        default=False,
        help_text=_("The learner's course completion status transmitted to Canvas.")
    )

    completed_timestamp = models.CharField(
        max_length=10,
        help_text=_(
            'Represents the canvas representation of a timestamp: yyyy-mm-dd, '
            'which is always 10 characters.'
        )
    )

    # Request-related information.
    status = models.CharField(max_length=100, blank=False, null=False)
    error_message = models.TextField(blank=True)
    created = models.DateTimeField(auto_now_add=True)
    grade = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        app_label = 'canvas'

    def __str__(self):
        return (
            '<CanvasLearnerDataTransmissionAudit {transmission_id} for enterprise enrollment {enrollment}, '
            'email {canvas_user_email}, and course {course_id}>'.format(
                transmission_id=self.id,
                enrollment=self.enterprise_course_enrollment_id,
                canvas_user_email=self.canvas_user_email,
                course_id=self.course_id
            )
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    def serialize(self, *args, **kwargs):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.

        # TODO: When we refactor to use a serialization flow consistent with how course metadata
        # is serialized, remove the serialization here and make the learner data exporter handle the work.
        """
        return json.dumps(self._payload_data(), sort_keys=True)

    def _payload_data(self):
        """
        Convert the audit record's fields into Canvas key/value pairs.
        """
        return dict(
            userID=self.canvas_user_email,
            courseID=self.course_id,
            courseCompleted="true" if self.course_completed else "false",
            completedTimestamp=self.completed_timestamp,
            grade=self.grade,
        )



"""
Base Integrated Channel application for specific integrated channels to use as a starting point.
"""

__version__ = "0.1.0"

default_app_config = ("integrated_channels.integrated_channel.apps."
                      "IntegratedChannelConfig")



"""
Mixins for Integrated Channel API Viewsets
"""
from uuid import UUID

from edx_rbac.mixins import PermissionRequiredForListingMixin
from rest_framework.exceptions import ParseError

from enterprise.constants import ENTERPRISE_ADMIN_ROLE
from enterprise.models import SystemWideEnterpriseUserRoleAssignment


class PermissionRequiredForIntegratedChannelMixin(PermissionRequiredForListingMixin):
    """
    `configuration_model` (class) - The RoleAssignmentClass against which DB-defined access is checked.
    """
    # fields that control permissions for 'list' actions
    list_lookup_field = 'enterprise_customer'
    allowed_roles = [ENTERPRISE_ADMIN_ROLE, ]
    role_assignment_class = SystemWideEnterpriseUserRoleAssignment

    @property
    def requested_enterprise_uuid(self):
        enterprise_customer_uuid = self.request.query_params.get('enterprise_customer')
        if not enterprise_customer_uuid:
            return None
        try:
            return UUID(enterprise_customer_uuid)
        except ValueError:
            raise ParseError('{} is not a valid uuid.'.format(enterprise_customer_uuid))

    def get_permission_object(self):
        """
        Used for "retrieve" actions. Determines the context (enterprise UUID) to check
        against for role-based permissions.
        """
        enterprise_customer_uuid = self.request.data.get('enterprise_customer_uuid')
        if enterprise_customer_uuid:
            return UUID(enterprise_customer_uuid)
        elif self.requested_enterprise_uuid:
            return self.requested_enterprise_uuid
        return None

    @property
    def base_queryset(self):
        """
        Required by the `PermissionRequiredForListingMixin`.
        For non-list actions, this is what's returned by `get_queryset()`.
        For list actions, some non-strict subset of this is what's returned by `get_queryset()`.
        """
        if not self.configuration_model:
            raise NotImplementedError
        kwargs = {}
        if self.requested_enterprise_uuid:
            kwargs.update({'enterprise_customer_id': self.requested_enterprise_uuid})
        return self.configuration_model.objects.filter(**kwargs)



# Generated by Django 2.2.24 on 2021-11-19 22:25

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0145_auto_20211013_1018'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalEnterpriseCustomerInviteKey',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('is_removed', models.BooleanField(default=False)),
                ('uuid', models.UUIDField(db_index=True, default=uuid.uuid4, editable=False)),
                ('usage_limit', models.PositiveIntegerField(default=10000, help_text='The number of times this key can be used to link a learner.')),
                ('expiration_date', models.DateTimeField(blank=True, help_text='The key will no longer be valid after this date.', null=True)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('enterprise_customer', models.ForeignKey(blank=True, db_constraint=False, help_text='The enterprise that can be linked using this key.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomer')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical enterprise customer invite key',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='EnterpriseCustomerInviteKey',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('is_removed', models.BooleanField(default=False)),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('usage_limit', models.PositiveIntegerField(default=10000, help_text='The number of times this key can be used to link a learner.')),
                ('expiration_date', models.DateTimeField(blank=True, help_text='The key will no longer be valid after this date.', null=True)),
                ('enterprise_customer', models.ForeignKey(help_text='The enterprise that can be linked using this key.', on_delete=django.db.models.deletion.CASCADE, related_name='invite_keys', to='enterprise.EnterpriseCustomer')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='enterprisecustomeruser',
            name='invite_key',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='linked_enterprise_customer_users', to='enterprise.EnterpriseCustomerInviteKey'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomeruser',
            name='invite_key',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomerInviteKey'),
        ),
    ]



"""
Exceptions for use by enterprise heartbeat module.
"""


class ServiceNotAvailable(Exception):
    """
    All heartbeat service status exception inherit from this excepton.
    """
    message = ''
    service_name = ''

    def __init__(self, message, service_name, *args):
        super().__init__(*args)
        self.message = message
        self.service_name = service_name


class EcommerceNotAvailable(ServiceNotAvailable):
    """
    Raised when E-Commerce service is not available.
    """
    service_name = 'E-Commerce'

    def __init__(self, message, *args):
        super().__init__(message, self.service_name, *args)


class DiscoveryNotAvailable(ServiceNotAvailable):
    """
    Raised when Course Discovery service is not available.
    """
    service_name = 'Course Discovery'

    def __init__(self, message, *args):
        super().__init__(message, self.service_name, *args)


class LMSNotAvailable(ServiceNotAvailable):
    """
    Raised when Learning Management System (LMS) service is not available.
    """
    service_name = 'Learning Management System (LMS)'

    def __init__(self, message, *args):
        super().__init__(message, self.service_name, *args)


class EnterpriseCatalogNotAvailable(ServiceNotAvailable):
    """
    Raised when Enterprise Catalog service is not available.
    """
    service_name = 'Enterprise Catalog'

    def __init__(self, message, *args):
        super().__init__(message, self.service_name, *args)



# Generated by Django 2.2.20 on 2021-04-07 20:04

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0126_auto_20210308_1522'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecatalogquery',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, null=True),
        ),
    ]



# Generated by Django 2.2.20 on 2021-07-07 13:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0007_auto_20210222_2225'),
    ]

    operations = [
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
    ]



# Generated by Django 2.2.20 on 2021-05-03 18:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0129_enterprisecatalogquery_uuid_unique'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text='Checked by default. When unchecked, learners in organizations with an integrated channel (LMS) will not see the "Find a Course" option in the enterprise learner portal.', verbose_name='Enable learner portal search for LMS customers'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text='Checked by default. When unchecked, learners in organizations with an integrated channel (LMS) will not see the "Find a Course" option in the enterprise learner portal.', verbose_name='Enable learner portal search for LMS customers'),
        ),
    ]
